/*
 * Licensed to Elasticsearch under one or more contributor
 * license agreements. See the NOTICE file distributed with
 * this work for additional information regarding copyright
 * ownership. Elasticsearch licenses this file to you under
 * the Apache License, Version 2.0 (the "License"); you may
 * not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package co.elastic.blog.elasticsearch.example;

import co.elastic.blog.elasticsearch.example.realm.WebStoreRealm;
import co.elastic.blog.elasticsearch.example.roles.WebStoreRolesProvider;
import org.elasticsearch.action.ActionListener;
import org.elasticsearch.common.settings.Setting;
import org.elasticsearch.common.settings.Settings;
import org.elasticsearch.watcher.ResourceWatcherService;
import org.elasticsearch.xpack.core.security.SecurityExtension;
import org.elasticsearch.xpack.core.security.authc.Realm;
import org.elasticsearch.xpack.core.security.authz.RoleDescriptor;

import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.function.BiConsumer;

/**
 * A {@link SecurityExtension} for our eCommerce platform
 *
 * @see "META-INF/services/org.elasticsearch.xpack.core.security.SecurityExtension"
 */
public class WebStoreSecurityExtension implements SecurityExtension {

    /**
     * Defines a single custom realm of type {@link WebStoreRealm#TYPE}.
     */
    @Override
    public Map<String, Realm.Factory> getRealms(ResourceWatcherService resourceWatcherService) {
        return Collections.singletonMap(WebStoreRealm.TYPE, config -> new WebStoreRealm(config, resourceWatcherService));
    }

    /**
     * Defines the settings for the  {@link WebStoreRealm}.
     */
    @Override
    public Map<String, Set<Setting<?>>> getRealmSettings() {
        return Collections.singletonMap(WebStoreRealm.TYPE, WebStoreRealm.getSettings());
    }

    /**
     * Defines a single roles provider method
     * @see WebStoreRolesProvider#lookup(Set, ActionListener)
     */
    @Override
    public List<BiConsumer<Set<String>, ActionListener<Set<RoleDescriptor>>>>
    getRolesProviders(Settings settings, ResourceWatcherService resourceWatcherService) {
        final WebStoreRolesProvider rolesProvider = new WebStoreRolesProvider();
        return Collections.singletonList(rolesProvider::lookup);
    }
}
