/*
 * Licensed to Elasticsearch under one or more contributor
 * license agreements. See the NOTICE file distributed with
 * this work for additional information regarding copyright
 * ownership. Elasticsearch licenses this file to you under
 * the Apache License, Version 2.0 (the "License"); you may
 * not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */


package co.elastic.blog.elasticsearch.example.realm;

import org.elasticsearch.xpack.core.security.authc.AuthenticationToken;

import java.security.GeneralSecurityException;

/**
 * An {@link AuthenticationToken} that presents a signed claim for a user in our eCommerce platform.
 */
public class WebStoreToken implements AuthenticationToken {

    private final String claimJson;
    private final WebStoreClaim claim;
    private byte[] signature;

    public WebStoreToken(String claimJson, byte[] signature) {
        this.claimJson = claimJson;
        this.claim = WebStoreClaim.parseFromJson(claimJson);
        this.signature = signature;
    }

    @Override
    public String principal() {
        return claim.getPrincipal();
    }

    @Override
    public byte[] credentials() {
        return signature;
    }

    @Override
    public void clearCredentials() {
        signature = null;
    }

    /**
     * @return The claims object, if-and-only-if the signature can be verified.
     */
    public WebStoreClaim getClaim(SignatureVerifier verifier) throws GeneralSecurityException {
        verifier.verify(claimJson, signature);
        return claim;
    }
}
